/**
 * Copyright (C) 2009-2015 Tatjana (Tanja) Kostic
 * <p>
 * This file belongs to jCleanCim, a tool supporting tasks of UML model managers for IEC TC57 CIM
 * and 61850 models.
 * <p>
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License version 3.
 * <p>
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details. You should have received a copy of the GNU Lesser
 * General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package org.tanjakostic.jcleancim.docgen;

import java.io.IOException;

import org.tanjakostic.jcleancim.common.Config;
import org.tanjakostic.jcleancim.docgen.collector.DocCollector;
import org.tanjakostic.jcleancim.docgen.collector.FreeFormDocumentation;
import org.tanjakostic.jcleancim.docgen.writer.Writer;
import org.tanjakostic.jcleancim.docgen.writer.word.WordWriterInput;
import org.tanjakostic.jcleancim.docgen.writer.word.doc.DocWordWriter;
import org.tanjakostic.jcleancim.docgen.writer.xml.WAXWriter;
import org.tanjakostic.jcleancim.docgen.writer.xml.WAXWriterInput;
import org.tanjakostic.jcleancim.util.Util;

/**
 * Factory class, allowing us to specify creation of concrete writers in one place and avoid
 * undesired dependencies.
 *
 * @author tatjana.kostic@ieee.org
 * @version $Id: WriterFactory.java 2733 2015-05-19 11:24:45Z CHTAKOS $
 */
public class WriterFactory {

	private WriterFactory() {
		// prevents creation
	}

	/**
	 * Creates writer initialised with <code>input</code> data.
	 *
	 * @throws UnsupportedInputFormatException
	 *             if the requested format (extension) of the input file(s) is not supported.
	 * @throws UnsupportedOutputFormatException
	 *             if the requested format (extension) of the output file(s) is not supported.
	 * @throws IOException
	 *             on any file system-related problem.
	 */
	public static Writer createWriter(Config cfg, DocCollector collector)
			throws UnsupportedInputFormatException, UnsupportedOutputFormatException, IOException {
		Util.ensureNotNull(cfg, "cfg");

		String retainedPath = "";

		// FIXME: see whether possible to refactor to something more elegant...

		String outXmlSpecPath = cfg.getDocgenXmlOutSpecFileAbsPath();
		if (outXmlSpecPath != null) {
			for (String ext : WAXWriter.FILE_EXTENSIONS) {
				if (outXmlSpecPath.endsWith(ext)) {
					WAXWriterInput input = new WAXWriterInput(cfg,
							collector.getFixedFormDocumentation());
					return new WAXWriter(input);
				}
			}
			retainedPath = outXmlSpecPath;
		}

		String outFilePath = cfg.getDocgenWordOutDocumentFileAbsPath();
		if (outFilePath != null) {
			FreeFormDocumentation freeFormDocumentation = collector.getFreeFormDocumentation();
			// for (String ext : DocxWordWriter.FILE_EXTENSIONS) {
			// // for .docx, use it only if .doc (binary) not explicitly specified:
			// if (outFilePath.endsWith(ext) && !cfg.isDocgenWordUseDocFormat()) {
			// WordWriterInput input = new WordWriterInput(cfg, freeFormDocumentation);
			// return new DocxWordWriter(input);
			// }
			// }
			for (String ext : DocWordWriter.FILE_EXTENSIONS) {
				if (outFilePath.endsWith(ext)) {
					WordWriterInput input = new WordWriterInput(cfg, freeFormDocumentation);
					return new DocWordWriter(input);
				}
			}
			retainedPath = outFilePath;
		}

		throw new UnsupportedOutputFormatException("Format (extension) of the output file '"
				+ retainedPath + "' not supported.");
	}
}
