/**
 * Copyright (C) 2009-2015 Tatjana (Tanja) Kostic
 * <p>
 * This file belongs to jCleanCim, a tool supporting tasks of UML model managers for IEC TC57 CIM
 * and 61850 models.
 * <p>
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License version 3.
 * <p>
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details. You should have received a copy of the GNU Lesser
 * General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package org.tanjakostic.jcleancim.docgen.collector;

import java.util.List;

/**
 * Data required for documentation of classes.
 * <p>
 * Here the layout you may use:
 *
 * <pre>
 * getHeadingText()
 *     getDescription()
 *     getDiagramDocs()        // loop and create figures
 *     getAttributesDoc()      // if getAttributesDoc().notEmpty() then create table
 *     getAssociationEndsDoc() // if getAssociationEndsDoc().notEmpty() then create table
 *     getOperationsDoc()      // if getOperationsDoc().notEmpty() then create table
 * </pre>
 *
 * @author tatjana.kostic@ieee.org
 * @version $Id: ClassDoc.java 2728 2015-05-14 20:00:03Z CHTAKOS $
 */
public interface ClassDoc extends ObjectDoc {
	public static final String CLASS_TITLE_PREFIX_FMT_CIM = "(%s) ";
	public static final String CLASS_TITLE_PREFIX_FMT_IEC61850 = "<<%s>> ";

	/** (deprecated, abstract), name, (kindLabel) */
	public String NO_ALIAS_HEADING_FORMAT = "%s%s%s";

	/** (deprecated, abstract), alias, nbsp, name */
	public String LN_HEADING_FORMAT = "%sLN: %s%sName: %s";

	/** (deprecated, abstract), alias, name, kindLabel */
	public String PRIM_DA_HEADING_FORMAT = "%s%s (%s %s)";

	/** (deprecated, abstract), alias, name */
	public String ANY_DA_OR_CDC_HEADING_FORMAT = "%s%s (%s)";

	/** (deprecated, abstract), alias, name, (stereotype) */
	public String OTHER_WITH_ALIAS_HEADING_FORMAT = "%s%s (%s%s)";

	/** Text to append to description if conditions need to be explicitly printed. */
	public static final String CONDITIONS_TEXT = "Conditions:";

	/**
	 * Returns the documentation for all the attributes of this class, suitable to be printed as a
	 * table. Use {@link PropertiesDoc#notEmpty()} to see whether there is anything to print.
	 */
	public PropertiesDoc getAttributesDoc();

	/**
	 * Returns the documentation for all the association ends on the 'other' side of this class,
	 * suitable to be printed as a table. Use {@link PropertiesDoc#notEmpty()} to see whether there
	 * is anything to print.
	 */
	public PropertiesDoc getAssocEndsDoc();

	/**
	 * Returns the documentation for all the operations of this class, suitable to be printed as a
	 * table. Use {@link PropertiesDoc#notEmpty()} to see whether there is anything to print.
	 */
	public PropertiesDoc getOperationsDoc();

	/**
	 * Returns the documentation for all the diagrams of this class.
	 */
	public List<FigureDoc> getDiagramDocs();
}
