/**
 * Copyright (C) 2009-2015 Tatjana (Tanja) Kostic
 * <p>
 * This file belongs to jCleanCim, a tool supporting tasks of UML model managers for IEC TC57 CIM
 * and 61850 models.
 * <p>
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License version 3.
 * <p>
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details. You should have received a copy of the GNU Lesser
 * General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package org.tanjakostic.jcleancim.docgen.collector;

import java.util.Map;

/**
 * Documentation in the free form, such as for printing content simply per package name (like for MS
 * Word documentation). The scope of retained packages is limitted according to configuration.
 *
 * @author tatjana.kostic@ieee.org
 * @version $Id: FreeFormDocumentation.java 2728 2015-05-14 20:00:03Z CHTAKOS $
 */
public class FreeFormDocumentation {

	private final ModelFinder _modelFinder;
	private final Map<String, PackageDoc> _packageDocs;

	/**
	 * Constructor.
	 *
	 * @param modelFinder
	 *            model facade; if null, most of placeholders will have error.
	 * @param packageDocs
	 *            "flattened" map of package documentation instances, with package name as key (to
	 *            allow to quickly find the package name from what is read in the placeholder). If
	 *            null or empty, placeholders dealing with packages will all have error and empty
	 *            content.
	 */
	public FreeFormDocumentation(ModelFinder modelFinder, Map<String, PackageDoc> packageDocs) {
		_modelFinder = modelFinder;
		_packageDocs = packageDocs;
	}

	public ModelFinder getModelFinder() {
		return _modelFinder;
	}

	public Map<String, PackageDoc> getPackageDocs() {
		return _packageDocs;
	}

	public PackageDoc getPackageDoc(String name) {
		return _packageDocs.get(name);
	}
}
