/**
 * Copyright (C) 2009-2015 Tatjana (Tanja) Kostic
 * <p>
 * This file belongs to jCleanCim, a tool supporting tasks of UML model managers for IEC TC57 CIM
 * and 61850 models.
 * <p>
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License version 3.
 * <p>
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details. You should have received a copy of the GNU Lesser
 * General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package org.tanjakostic.jcleancim.docgen.collector.impl;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.log4j.Logger;
import org.tanjakostic.jcleancim.docgen.collector.DocgenConfig;
import org.tanjakostic.jcleancim.docgen.collector.TableSpec;
import org.tanjakostic.jcleancim.docgen.collector.WAX;
import org.tanjakostic.jcleancim.model.TextDescription;
import org.tanjakostic.jcleancim.model.UmlAttribute;
import org.tanjakostic.jcleancim.model.UmlClass;

/**
 * Documentation for attributes in simple format.
 *
 * @author tatjana.kostic@ieee.org
 * @version $Id: DefaultAttributesDoc.java 2728 2015-05-14 20:00:03Z CHTAKOS $
 */
class DefaultAttributesDoc extends AbstractPropertiesDoc {
	private static final Logger _logger = Logger.getLogger(DefaultAttributesDoc.class.getName());

	public static final String INTRO_FMT = " shows all attributes of %s.";
	public static final String CAPTION_FMT = "Attributes of %s";

	/**
	 * Constructor.
	 */
	public DefaultAttributesDoc(DocgenConfig docgenCfg, UmlClass c) {
		super(docgenCfg, c, "att", String.format(INTRO_FMT, c.getName()), String.format(
				CAPTION_FMT, c.getQualifiedName()), TableSpec.ATTRS, null);
		log(_logger, "---- collecting doc for attributes of " + c.getName() + " ...");

		Collection<UmlAttribute> retainedNatives = new ArrayList<UmlAttribute>();
		Collection<UmlAttribute> retainedInheriteds = new ArrayList<UmlAttribute>();
		super.filterAttributes(c, retainedNatives, retainedInheriteds);

		for (UmlAttribute a : retainedNatives) {
			String descPrefix = getDeprecatedTextAsPrefix(a) + getInitValAsPrefix(a);
			CellText desc = deduceCellText(descPrefix, a);

			EntryDocImpl entry = EntryDocImpl.createData(desc.formatInfo, a.getName(), a
					.getMultiplicity().getBounds(), a.getType().getName(), desc.text);
			addEntry(entry);

			// for xml:
			initRawData(entry, a);
			initAttributeRawData(entry, a);
			entry.putCell("desc", a.getHtmlDescription().prepend(getInitValAsPrefix(a), a).text);
		}
		for (UmlAttribute a : retainedInheriteds) {
			String descPrefix = getDeprecatedTextAsPrefix(a);
			TextDescription docInh = new TextDescription(INHERITED_FROM
					+ a.getContainingClass().getName());
			CellText desc = deduceCellText(descPrefix, docInh, null, a);

			EntryDocImpl entry = EntryDocImpl.createData(null, a.getName(), a.getMultiplicity()
					.getBounds(), a.getType().getName(), desc.text);
			addEntry(entry);

			// for xml:
			initRawData(entry, a, a.getContainingClass().getName());
			initAttributeRawData(entry, a);
		}

		log(_logger, ">>>>>> " + toString());
	}

	private static void initAttributeRawData(EntryDocImpl entry, UmlAttribute a) {
		entry.putCell("mult", a.getMultiplicity().getBounds());
		entry.putCell(WAX.A_type, a.getType().getName());
		entry.putCell("defaultValue", a.getInitValue());
	}
}
