/**
 * Copyright (C) 2009-2015 Tatjana (Tanja) Kostic
 * <p>
 * This file belongs to jCleanCim, a tool supporting tasks of UML model managers for IEC TC57 CIM
 * and 61850 models.
 * <p>
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License version 3.
 * <p>
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details. You should have received a copy of the GNU Lesser
 * General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package org.tanjakostic.jcleancim.docgen.writer.word.docx;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.tanjakostic.jcleancim.docgen.UnsupportedInputFormatException;
import org.tanjakostic.jcleancim.docgen.UnsupportedOutputFormatException;
import org.tanjakostic.jcleancim.docgen.collector.PropertiesDoc;
import org.tanjakostic.jcleancim.docgen.writer.Caption;
import org.tanjakostic.jcleancim.docgen.writer.Caption.CaptionKind;
import org.tanjakostic.jcleancim.docgen.writer.Range;
import org.tanjakostic.jcleancim.docgen.writer.Style;
import org.tanjakostic.jcleancim.docgen.writer.word.AbstractWordWriter;
import org.tanjakostic.jcleancim.docgen.writer.word.WordPatternFinder;
import org.tanjakostic.jcleancim.docgen.writer.word.WordWriterInput;
import org.tanjakostic.jcleancim.util.Util;

/**
 * @author tatjana.kostic@ieee.org
 * @version $Id: DocxWordWriter.java 2733 2015-05-19 11:24:45Z CHTAKOS $
 */
public class DocxWordWriter extends AbstractWordWriter<Object> {
	private static final Logger _logger = Logger.getLogger(DocxWordWriter.class.getName());

	// private WordprocessingMLPackage _wordDoc;

	/**
	 * Constructor.
	 *
	 * @param input
	 * @throws UnsupportedInputFormatException
	 *             if configured with template in unsupported format
	 * @throws UnsupportedOutputFormatException
	 *             if configured with output in unsupported format
	 * @throws IOException
	 */
	public DocxWordWriter(WordWriterInput input) throws UnsupportedInputFormatException,
			UnsupportedOutputFormatException, IOException {
		super(input);
	}

	// FIXME ------------------------------------
	private int doInsertTable(Range<Object> range, PropertiesDoc doc, Style tabhead) {
		return 0;
	}

	// FIXME end - from here below, we're more or less in synch with DocWordWriter

	// ===== Impl. of org.tanjakostic.jcleancim.docgen.writer.word.WordHelper methods =====

	// ----------------- document lifecycle -----------------

	@Override
	public String getWordAppName() {
		throw new RuntimeException("Not implemented yet.");
		// return getExtProperties().getApplication();
	}

	@Override
	public String getWordAppVersion() {
		throw new RuntimeException("Not implemented yet.");
		// return getExtProperties().getAppVersion();
	}

	@Override
	public void openDoc() throws IOException {
		_logger.info("  opening MSWord file '" + getInput().getOutFilePath() + "'");
		throw new RuntimeException("Not implemented yet.");
		// try {
		// _wordDoc = WordprocessingMLPackage.load(new File(getInput().getOutFilePath()));
		// } catch (Docx4JException e) {
		// throw new IOException(e);
		// }
	}

	@Override
	public void closeDoc() throws IOException {
		throw new RuntimeException("Not implemented yet.");
		// if (_wordDoc != null) {
		// String outFilePath = getInput().getOutFilePath();
		// _logger.info("closing MSWord file '" + outFilePath + "'");
		// SaveToZipFile saver = new SaveToZipFile(_wordDoc);
		// try {
		// saver.save(outFilePath);
		// } catch (Docx4JException e) {
		// throw new IOException(e);
		// }
		// }
	}

	// --------- styles -------------

	@Override
	public Map<String, String> getExistingStyleNames() {
		throw new RuntimeException("Not implemented yet.");
		// Map<String, String> result = new HashMap<String, String>();
		// Styles styles =
		// _wordDoc.getMainDocumentPart().getStyleDefinitionsPart().getJaxbElement();
		// for (org.docx4j.wml.Style style : styles.getStyle()) {
		// result.put(style.getName().getVal(), style.getStyleId());
		// }
		// return result;
	}

	// ---------------- TOCs ---------------

	/**
	 * {@inheritDoc}
	 * <p>
	 * This is no-op: you need to open the document in MS Word.
	 */
	@Override
	public void updateFields() {
		// no-iop
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * This is no-op: you need to open the document in MS Word.
	 */
	@Override
	public void updateTablesOf(String what) {
		// no-iop
	}

	// ----------------------- custom properties -----------------------

	@Override
	public Map<String, String> getCustomDocProperties() {
		throw new RuntimeException("Not implemented yet.");
		// org.docx4j.docProps.custom.Properties props = getCustomStringProperties();
		// Map<String, String> result = new HashMap<String, String>();
		// for (org.docx4j.docProps.custom.Properties.Property prop : props.getProperty()) {
		// if (prop.getLpwstr() != null) {
		// result.put(prop.getName(), prop.getLpwstr());
		// }
		// }
		// return result;
	}

	@Override
	public void setCustomDocProperties(Map<String, String> newCustomProps) {
		throw new RuntimeException("Not implemented yet.");
		// Map<String, String> newProps = new HashMap<String, String>(newCustomProps);
		// org.docx4j.docProps.custom.Properties customProps = getCustomStringProperties();
		// for (org.docx4j.docProps.custom.Properties.Property customProp :
		// customProps.getProperty()) {
		// String name = customProp.getName();
		// // "move" from requested map into Word document:
		// if (newProps.containsKey(name)) {
		// String value = newProps.remove(name);
		// _logger.debug(String.format("setting existing custom property: %s = %s ", name,
		// value));
		// setCustomStringProperty(customProp, name, value);
		// }
		// }
		// if (newProps.isEmpty()) {
		// // all requested props existed in the doc and their new values have been set
		// return;
		// }
		//
		// org.docx4j.docProps.custom.ObjectFactory of = new
		// org.docx4j.docProps.custom.ObjectFactory();
		//
		// for (Map.Entry<String, String> newProp : newProps.entrySet()) {
		// String name = newProp.getKey();
		// String value = newProp.getValue();
		// _logger.info(String.format("adding new custom property: %s = %s ", name, value));
		//
		// org.docx4j.docProps.custom.Properties.Property newCustomProp = of
		// .createPropertiesProperty();
		// setCustomStringProperty(newCustomProp, name, value);
		// customProps.getProperty().add(newCustomProp);
		// }
	}

	// --------------------- cursors ---------------------

	@Override
	public Map<CaptionKind, List<Caption<Object>>> collectCaptions() {
		throw new RuntimeException("Not implemented yet.");
		// Map<CaptionKind, List<Caption<Object>>> result = new LinkedHashMap<CaptionKind,
		// List<Caption<Object>>>();
		// List<Caption<Object>> figures = new ArrayList<Caption<Object>>();
		// result.put(CaptionKind.Figure, figures);
		// List<Caption<Object>> tables = new ArrayList<Caption<Object>>();
		// result.put(CaptionKind.Table, tables);
		//
		// List<Object> paras = getAllElementFromObject(_wordDoc.getMainDocumentPart(),
		// org.docx4j.wml.P.class);
		//
		// for (Object para : paras) {
		// org.docx4j.wml.P paraEl = (org.docx4j.wml.P) para;
		//
		// String styleId = getParagraphStyleId(paraEl);
		// Range<Object> range = createRange(paraEl);
		// String text = range.getText(); // FIXME: impl. here
		//
		// if (CaptionKind.looksLikeFigureCaption(styleId, text)) {
		// Caption<Object> caption = createCaption(CaptionKind.Figure, range);
		// _logger.info("   found '" + text + "'");
		// figures.add(caption);
		// } else if (CaptionKind.looksLikeTableCaption(styleId, text)) {
		// Caption<Object> caption = createCaption(CaptionKind.Table, range);
		// _logger.info("   found '" + text + "'");
		// tables.add(caption);
		// }
		// }
		//
		// return result;
	}

	@Override
	public Range<Object> createRange(Object object) {
		return new DocxWordRange(object);
	}

	@Override
	public WordPatternFinder<Object> createPatternFinder() {
		throw new RuntimeException("Not implemented yet.");
		// TODO Auto-generated method stub

		// // if pattern common to both Doc and Docx, refactor to move to AbstractWordWriter
		// String msPattern = Placeholder.MS_PATTERN;
		// return new DocxWordPatternFinder(_wordDoc, msPattern);
	}

	// ---------------- actual writing into output document -------------------

	// FIXME here!

	@Override
	public Range<Object> getDocumentAsRange() {
		throw new RuntimeException("Not implemented yet.");
		// return createRange(getDocumentContent());
	}

	@Override
	public Range<Object> duplicateRange(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// return createRange(Dispatch.call(range.getObject(), "Duplicate").toDispatch());
	}

	@Override
	public void prependText(Range<Object> range, String newText) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "InsertBefore", newText);
	}

	@Override
	public int getRangeParagraphOutlineLevel(Range<Object> range, int paraIdx) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch paragraph = getParagraph(range.getObject(), paraIdx);
		// if (paragraph == null) {
		// return -1;
		// }
		// return Dispatch.get(paragraph, "OutlineLevel").getInt();
	}

	@Override
	public String appendRawTextInNewParagraphWithStyle(Range<Object> range, String newText,
			Style style) {
		appendNewLine(range);
		return appendTextWithStyle(range, newText, style);
	}

	@Override
	public String appendHtmlTextInNewParagraphWithStyle(Range<Object> range, String newMarkup,
			Style style) {
		return appendRawTextInNewParagraphWithStyle(range, newMarkup, style);
	}

	// mainDocumentPart.addStyledParagraphofText(styleId, "My Text");

	@Override
	public String appendTextWithStyle(Range<Object> range, String newText, Style style) {
		collapseRangeToEnd(range);
		// setRangeStyle(range.getObject(), style);
		return appendText(range, newText);
	}

	// ------ inserting a figure/table, its caption, and references to caption ------

	@Override
	public void insertFigureCaption(Range<Object> range, String text, String logMsg) {
		insertCaption(range, CaptionKind.Figure, false, text, logMsg);

		// int paraCount = 2;
		// Object.call(range.getObject(), "MoveEnd", wdParagraph, getVariant(paraCount));
		//
		// Object secondPara = getParagraph(range.getObject(), paraCount);
		// setRangeStyle(secondPara, Style.figcapt);
		//
		// int endMinusOne = range.getEnd() - 1;
		// range.setEnd(endMinusOne);
		// collapseRangeToEnd(range);
	}

	@Override
	public void insertTableCaption(Range<Object> range, String text, String logMsg) {
		throw new RuntimeException("Not implemented yet.");
		// Range<Object> tableRange = getFirstTableAsRange(range.getObject());
		//
		// insertCaption(tableRange, CaptionKind.Table, true, text, logMsg);
		//
		// Object firstPara = getParagraph(tableRange.getObject(), 1);
		// setRangeStyle(firstPara, Style.tabcapt);

		// collapseRangeToEnd(range);
		//
		// firstPara = getParagraph(range.getObject(), 1);
		// // last style was tabcell, set by insertTable, using keepWithNext, reassign para style
		// setRangeStyle(firstPara, Style.para);
	}

	private static void insertCaption(Range<Object> range, CaptionKind label,
			boolean positionAbove, String text, String logMsg) {
		long t0 = System.currentTimeMillis();

		String textAfterCaptionNumber = " " + Util.EN_DASH + " " + text;

		// Dispatch.call(range.getObject(), "InsertCaption", label,
		// new Variant(textAfterCaptionNumber), VARIANT_EMPTY_STRING, position, getVariant(0));

		_logger.info("... " + (System.currentTimeMillis() - t0) + " ms for insertCaption(): "
				+ logMsg + textAfterCaptionNumber);
	}

	@Override
	public void insertFigureRef(Range<Object> insertPointRange, int figNumber) {
		insertCaptionRef(insertPointRange, CaptionKind.Figure, figNumber);
	}

	@Override
	public void insertTableRef(Range<Object> insertPointRange, int tabNumber) {
		insertCaptionRef(insertPointRange, CaptionKind.Table, tabNumber);
	}

	private void insertCaptionRef(Range<Object> range, CaptionKind kind, int idx) {
		long t0 = System.currentTimeMillis();

		prependText(range, " ");
		collapseRangeToStart(range);

		// TODO
		_logger.info("... " + (System.currentTimeMillis() - t0) + " ms for insertCaptionRef()");
		_logger.info("--------------------------------------------------");
	}

	@Override
	public void insertFigure(Range<Object> range, File pic) {
		throw new RuntimeException("Not implemented yet.");

		// range.setText("");

		// "Link to File" = false, "Save with Document" = true"
		//
		// Dispatch inlineShapes = Dispatch.call(range.getObject(), "InlineShapes").toDispatch();
		// Dispatch.call(inlineShapes, "AddPicture", pic.getAbsolutePath(), VARIANT_FALSE,
		// VARIANT_TRUE);

		// range.setEnd(range.getEnd() + 1);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns expanded range to the end of the table. We do not collapse original range here, as we
	 * assume the caption will need to be inserted before the table.
	 */
	@Override
	public int insertTable(Range<Object> range, PropertiesDoc doc, Style tabhead) {
		return doInsertTable(range, doc, tabhead);
	}

	// ------------------------------ tests only ---------------------------

	@Override
	public String getRangeParagraphStyleName(Range<Object> range, int paraIdx) {
		throw new RuntimeException("Not implemented yet.");
		// Object paragraph = getParagraph(range.getObject(), paraIdx);
		// return (paragraph == null) ? null : getParagraphStyleName(paragraph);
	}

	@Override
	public int getRangeParagraphCount(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch paragraphs = Dispatch.get(range.getObject(), "Paragraphs").toDispatch();
		// return getCount(paragraphs);
	}

	@Override
	public void prependNewLine(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "InsertParagraphBefore");
	}

	@Override
	public void appendNewLine(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "InsertParagraphAfter");
	}

	@Override
	public String appendTextInNewParagraph(Range<Object> range, String newText) {
		appendNewLine(range);
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "InsertAfter", newText);
		// return range.getText();
	}

	@Override
	public String appendText(Range<Object> range, String newText) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "InsertAfter", newText);
		// return range.getText();
	}

	@Override
	public boolean isRangeWithTable(Object range) {
		throw new RuntimeException("Not implemented yet.");
		// return null != getFirstTableAsRange(range);
	}

	@Override
	public void collapseRangeToEnd(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "Collapse", wdCollapseEnd);
	}

	private void collapseRangeToStart(Range<Object> range) {
		throw new RuntimeException("Not implemented yet.");
		// Dispatch.call(range.getObject(), "Collapse", wdCollapseStart);
	}

	// ===== Impl. of org.tanjakostic.jcleancim.docgen.writer.word.WordWriter methods =====

	@Override
	public boolean applyCloseReopen() {
		return false;
	}

	// ===== Impl. of org.tanjakostic.jcleancim.docgen.writer.Writer methods =====

	public static final List<String> FILE_EXTENSIONS = Arrays.asList(".docx");

	@Override
	public Set<String> getSupportedFormats() {
		return new HashSet<String>(FILE_EXTENSIONS);
	}
}
